<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\CsvImportTrait;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyMaklumatProjectRequest;
use App\Http\Requests\StoreMaklumatProjectRequest;
use App\Http\Requests\UpdateMaklumatProjectRequest;
use App\Models\MaklumatProject;
use Gate;
use Illuminate\Http\Request;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class MaklumatProjectController extends Controller
{
    use MediaUploadingTrait, CsvImportTrait;

    public function index(Request $request)
    {
        abort_if(Gate::denies('maklumat_project_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = MaklumatProject::query()->select(sprintf('%s.*', (new MaklumatProject)->table));
            $table = Datatables::of($query);

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate      = 'maklumat_project_show';
                $editGate      = 'maklumat_project_edit';
                $deleteGate    = 'maklumat_project_delete';
                $crudRoutePart = 'maklumat-projects';

                return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                ));
            });

            $table->editColumn('id', function ($row) {
                return $row->id ? $row->id : '';
            });
            $table->editColumn('lot', function ($row) {
                return $row->lot ? $row->lot : '';
            });
            $table->editColumn('mukim', function ($row) {
                return $row->mukim ? $row->mukim : '';
            });
            $table->editColumn('daerah', function ($row) {
                return $row->daerah ? MaklumatProject::DAERAH_SELECT[$row->daerah] : '';
            });
            $table->editColumn('keluasan', function ($row) {
                return $row->keluasan ? $row->keluasan : '';
            });
            $table->editColumn('kategori_penggunaan_tanah', function ($row) {
                return $row->kategori_penggunaan_tanah ? MaklumatProject::KATEGORI_PENGGUNAAN_TANAH_SELECT[$row->kategori_penggunaan_tanah] : '';
            });
            $table->editColumn('sekatan_sekatan_penting', function ($row) {
                return $row->sekatan_sekatan_penting ? $row->sekatan_sekatan_penting : '';
            });

            $table->editColumn('pelan_kunci', function ($row) {
                if (! $row->pelan_kunci) {
                    return '';
                }
                $links = [];
                foreach ($row->pelan_kunci as $media) {
                    $links[] = '<a href="' . $media->getUrl() . '" target="_blank"><img src="' . $media->getUrl('thumb') . '" width="50px" height="50px"></a>';
                }

                return implode(' ', $links);
            });
            $table->editColumn('pelan_lokasi', function ($row) {
                if (! $row->pelan_lokasi) {
                    return '';
                }
                $links = [];
                foreach ($row->pelan_lokasi as $media) {
                    $links[] = '<a href="' . $media->getUrl() . '" target="_blank"><img src="' . $media->getUrl('thumb') . '" width="50px" height="50px"></a>';
                }

                return implode(' ', $links);
            });

            $table->rawColumns(['actions', 'placeholder', 'pelan_kunci', 'pelan_lokasi']);

            return $table->make(true);
        }

        return view('admin.maklumatProjects.index');
    }

    public function create()
    {
        abort_if(Gate::denies('maklumat_project_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.maklumatProjects.create');
    }

    public function store(StoreMaklumatProjectRequest $request)
    {
        $maklumatProject = MaklumatProject::create($request->all());

        foreach ($request->input('pelan_kunci', []) as $file) {
            $maklumatProject->addMedia(storage_path('tmp/uploads/' . basename($file)))->toMediaCollection('pelan_kunci');
        }

        foreach ($request->input('pelan_lokasi', []) as $file) {
            $maklumatProject->addMedia(storage_path('tmp/uploads/' . basename($file)))->toMediaCollection('pelan_lokasi');
        }

        if ($media = $request->input('ck-media', false)) {
            Media::whereIn('id', $media)->update(['model_id' => $maklumatProject->id]);
        }

        return redirect()->route('admin.maklumat-projects.index');
    }

    public function edit(MaklumatProject $maklumatProject)
    {
        abort_if(Gate::denies('maklumat_project_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.maklumatProjects.edit', compact('maklumatProject'));
    }

    public function update(UpdateMaklumatProjectRequest $request, MaklumatProject $maklumatProject)
    {
        $maklumatProject->update($request->all());

        if (count($maklumatProject->pelan_kunci) > 0) {
            foreach ($maklumatProject->pelan_kunci as $media) {
                if (! in_array($media->file_name, $request->input('pelan_kunci', []))) {
                    $media->delete();
                }
            }
        }
        $media = $maklumatProject->pelan_kunci->pluck('file_name')->toArray();
        foreach ($request->input('pelan_kunci', []) as $file) {
            if (count($media) === 0 || ! in_array($file, $media)) {
                $maklumatProject->addMedia(storage_path('tmp/uploads/' . basename($file)))->toMediaCollection('pelan_kunci');
            }
        }

        if (count($maklumatProject->pelan_lokasi) > 0) {
            foreach ($maklumatProject->pelan_lokasi as $media) {
                if (! in_array($media->file_name, $request->input('pelan_lokasi', []))) {
                    $media->delete();
                }
            }
        }
        $media = $maklumatProject->pelan_lokasi->pluck('file_name')->toArray();
        foreach ($request->input('pelan_lokasi', []) as $file) {
            if (count($media) === 0 || ! in_array($file, $media)) {
                $maklumatProject->addMedia(storage_path('tmp/uploads/' . basename($file)))->toMediaCollection('pelan_lokasi');
            }
        }

        return redirect()->route('admin.maklumat-projects.index');
    }

    public function show(MaklumatProject $maklumatProject)
    {
        abort_if(Gate::denies('maklumat_project_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.maklumatProjects.show', compact('maklumatProject'));
    }

    public function destroy(MaklumatProject $maklumatProject)
    {
        abort_if(Gate::denies('maklumat_project_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $maklumatProject->delete();

        return back();
    }

    public function massDestroy(MassDestroyMaklumatProjectRequest $request)
    {
        $maklumatProjects = MaklumatProject::find(request('ids'));

        foreach ($maklumatProjects as $maklumatProject) {
            $maklumatProject->delete();
        }

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {
        abort_if(Gate::denies('maklumat_project_create') && Gate::denies('maklumat_project_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $model         = new MaklumatProject();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }
}
